@@ -34,7 +34,7 @@
 */
 
 #include "BTreePrivate.h"
-
+#include "hfs_endian.h"
 
 
 ///////////////////////// BTree Module Node Operations //////////////////////////
@@ -145,6 +145,9 @@ OSStatus	GetNode		(BTreeControlBlockPtr	 btreePtr,
 	}
 	++btreePtr->numGetNodes;
 	
+	SWAP_BT_NODE(nodePtr, (btreePtr->fcbPtr->fcbVolume->vcbSignature == kHFSPlusSigWord),
+			btreePtr->fcbPtr->fcbFileID, 0);
+
 	//
 	// Optimization
 	// Only call CheckNode if the node came from disk.
@@ -290,6 +293,12 @@ OSStatus	ReleaseNode	(BTreeControlBlockPtr	 btreePtr,
 	
 	if (nodePtr->buffer != nil)
 	{
+		/*
+		 * The nodes must remain in the cache as big endian!
+		 */
+		SWAP_BT_NODE(nodePtr, (btreePtr->fcbPtr->fcbVolume->vcbSignature == kHFSPlusSigWord),
+			btreePtr->fcbPtr->fcbFileID, 1);
+
 		releaseNodeProc = btreePtr->releaseBlockProc;
 		err = releaseNodeProc (btreePtr->fcbPtr,
 							   nodePtr,
@@ -307,6 +316,45 @@ OSStatus	ReleaseNode	(BTreeControlBlockPtr	 btreePtr,
 }
 
 
+/*-------------------------------------------------------------------------------
+
+Routine:	TrashNode	-	Call FS Agent to release node obtained by GetNode, and
+							not store it...mark it as bad.
+
+Function:	Informs the FS Agent that a BTree node may be released and thrown away.
+
+Input:		btreePtr		- pointer to BTree control block
+			nodeNum			- number of node to release
+						
+Result:		noErr		- success
+			!= noErr	- failure
+-------------------------------------------------------------------------------*/
+
+OSStatus	TrashNode	(BTreeControlBlockPtr	 btreePtr,
+						 NodePtr				 nodePtr )
+{
+	OSStatus			 err;
+	ReleaseBlockProcPtr	 releaseNodeProc;
+	
+
+	err = noErr;
+	
+	if (nodePtr->buffer != nil)
+	{
+		releaseNodeProc = btreePtr->releaseBlockProc;
+		err = releaseNodeProc (btreePtr->fcbPtr,
+							   nodePtr,
+							   kReleaseBlock | kTrashBlock );
+		PanicIf (err, "\TrashNode: releaseNodeProc returned error.");
+		++btreePtr->numReleaseNodes;
+	}
+
+	nodePtr->buffer			= nil;
+	nodePtr->blockHeader	= nil;
+	
+	return err;
+}
+
 
 /*-------------------------------------------------------------------------------
 
@@ -342,6 +390,8 @@ OSStatus	UpdateNode	(BTreeControlBlockPtr	 btreePtr,
 		}
 
 	//	LogStartTime(kTraceReleaseNode);
+		SWAP_BT_NODE(nodePtr, (btreePtr->fcbPtr->fcbVolume->vcbSignature == kHFSPlusSigWord),
+			btreePtr->fcbPtr->fcbFileID, 1);
 
 		releaseNodeProc = btreePtr->releaseBlockProc;
 		err = releaseNodeProc (btreePtr->fcbPtr,
@@ -470,8 +520,11 @@ static void PrintNode(const NodeDescPtr node, UInt16 nodeSize, UInt32 nodeNumber
 	lp = (UInt32*) node;
 	offset = 0;
 	
-	while (rows-- > 0)
-		printf("%04X: %08X %08X %08X %08X\n", (u_int)offset++, *lp++, *lp++, *lp++, *lp++);
+	while (rows-- > 0) {
+		printf( "%04X: %08X %08X %08X %08X\n", (u_int)offset++, 
+				*(lp + 0), *(lp + 1), *(lp + 2), *(lp + 3) );
+		lp += 4;
+	}
 }
 #endif
 
