@@ -2,24 +2,21 @@
  * Copyright (c) 2000-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ *
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
+ *
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
- * 
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
+ *
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -96,7 +93,7 @@ GetVolumeBlock (SVCB *volume, UInt64 blockNum, GetBlockOptions options, BlockDes
 
 		mdb = (HFSMasterDirectoryBlock *)block->buffer;
 		signature = SWAP_BE16(mdb->drSigWord);
-		if (signature == kHFSPlusSigWord )
+		if (signature == kHFSPlusSigWord || signature == kHFSXSigWord)
 			SWAP_HFSPLUSVH(block->buffer);
 		else if (signature == kHFSSigWord)
 			SWAP_HFSMDB(block->buffer);
@@ -130,7 +127,7 @@ ReleaseVolumeBlock (SVCB *volume, BlockDescriptor *block, ReleaseBlockOptions op
 	 */
 	if (!(options & kSkipEndianSwap)) {
 		signature = ((HFSMasterDirectoryBlock *)block->buffer)->drSigWord;
-		if (signature == kHFSPlusSigWord)
+		if (signature == kHFSPlusSigWord || signature == kHFSXSigWord)
 			SWAP_HFSPLUSVH(block->buffer);
 		else if (signature == kHFSSigWord)
 			SWAP_HFSMDB(block->buffer);
@@ -178,7 +175,7 @@ GetFileBlock (SFCB *file, UInt32 blockNum, GetBlockOptions options, BlockDescrip
 	offset = (SInt64) ((UInt64) diskBlock) << kSectorShift;
 
 	result = CacheRead (cache, offset, file->fcbBlockSize, &buffer);
-	if (result) return (result);
+	if (result)  return (result);
 
 	block->blockHeader = buffer;
 	block->buffer = buffer->Buffer;
