@@ -2,24 +2,21 @@
  * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ *
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
+ *
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
- * 
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
+ *
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -50,11 +47,12 @@ struct CatalogIterationSummary gCIS;
 SGlobPtr gScavGlobals;
 
 /* Local routines for checking catalog structures */
-static int  CheckCatalogRecord(const HFSPlusCatalogKey *key,
+static int  CheckCatalogRecord(SGlobPtr GPtr, const HFSPlusCatalogKey *key,
                                const CatalogRecord *rec, UInt16 reclen);
 static int  CheckCatalogRecord_HFS(const HFSCatalogKey *key,
                                    const CatalogRecord *rec, UInt16 reclen);
 
+static int  CheckIfAttributeExists(const UInt32 fileID); 
 static int  CheckDirectory(const HFSPlusCatalogKey * key, const HFSPlusCatalogFolder * dir);
 static int  CheckFile(const HFSPlusCatalogKey * key, const HFSPlusCatalogFile * file);
 static int  CheckThread(const HFSPlusCatalogKey * key, const HFSPlusCatalogThread * thread);
@@ -79,7 +77,6 @@ static 	OSErr	UniqueDotName( 	SGlobPtr GPtr,
                                 Boolean isSingleDotName,
                                 Boolean isHFSPlus );
 static Boolean 	FixDecomps(	u_int16_t charCount, const u_int16_t *inFilename, HFSUniStr255 *outFilename );
-static void 	PrintName( int theCount, const UInt8 *theNamePtr, Boolean isUnicodeString );
 
 /*
  * CheckCatalogBTree - Verifies the catalog B-tree structure
@@ -163,7 +160,7 @@ CheckCatalogBTree( SGlobPtr GPtr )
  * Called in leaf-order for every leaf record in the Catalog B-tree
  */
 static int
-CheckCatalogRecord(const HFSPlusCatalogKey *key, const CatalogRecord *rec, UInt16 reclen)
+CheckCatalogRecord(SGlobPtr GPtr, const HFSPlusCatalogKey *key, const CatalogRecord *rec, UInt16 reclen)
 {
 	int 						result = 0;
 	Boolean						isHFSPlus;
@@ -315,6 +312,50 @@ CheckCatalogRecord_HFS(const HFSCatalogKey *key, const CatalogRecord *rec, UInt1
 	return (result);
 }
 
+/* CheckIfAttributeExists - check if atleast one extend attribute
+ *                          exists for given file/folder ID 
+ * 
+ * Returns: 0 - if not extended attribute exists
+ *          1 - if atleast one extended attribute exists
+ *          n - some error occured 
+ */
+static int 
+CheckIfAttributeExists(const UInt32 fileID) 
+{
+	int result = 0;
+	AttributeKey *attrKey;
+	BTreeIterator iterator;
+
+	/* Initialize the iterator, note that other fields in iterator are initialized to zero */
+	ClearMemory(&iterator, sizeof(BTreeIterator));
+
+	/* Initialize attribute key to iterator key.  Other fields initialized to zero */
+	attrKey = (AttributeKey *)&iterator.key;
+	attrKey->keyLength = kAttributeKeyMinimumLength;
+	attrKey->cnid = fileID;
+
+	/* Search for attribute with NULL name.  This will place the iterator at correct fileID location in BTree */	
+	result = BTSearchRecord(gScavGlobals->calculatedAttributesFCB, &iterator, kInvalidMRUCacheKey, NULL, NULL, &iterator);
+	if (result && (result != btNotFound)) {
+#if 0 //DEBUG_XATTR
+		printf ("%s: No matching attribute record found\n", __FUNCTION__);
+#endif
+		goto out;
+	}
+
+	/* Iterate to next record and check there is a valid extended attribute for this file ID */ 
+	result = BTIterateRecord(gScavGlobals->calculatedAttributesFCB, kBTreeNextRecord, &iterator, NULL, NULL);
+	if ((result == noErr) && (attrKey->cnid == fileID)) {
+		/* Set success return value only if we did _find_ an attribute record for the current fileID */
+		result = 1;
+	} 
+out:
+#if 0 //DEBUG_XATTR
+	printf ("%s: Retval=%d for fileID=%d\n", __FUNCTION__, result, fileID); 
+#endif
+	return result;
+}
+
 /*
  * CheckDirectory - verify a catalog directory record
  *
@@ -329,11 +370,31 @@ CheckDirectory(const HFSPlusCatalogKey * key, const HFSPlusCatalogFolder * dir)
 
 	dirID = dir->folderID;
 
-	if (dir->flags != 0) {
+	if ((dir->flags & (kHFSFileLockedMask | kHFSThreadExistsMask)) != 0) {
 		RcdError(gScavGlobals, E_CatalogFlagsNotZero);
 		gScavGlobals->CBTStat |= S_ReservedNotZero;
 	}
 
+	/* 3971173, 3977448 : HFS does not clear the attribute bit after it deletes the 
+	 * last attribute.  Fix it later in 3977448 by removing the call for checking attrs
+	 */
+	result = CheckIfAttributeExists(dir->folderID);
+	if (result == 1) {
+		/* Atleast one extended attribute exists for this folder ID */
+		/* 3857929: Record if directory record has attribute and/or security bit set */
+		RecordXAttrBits(gScavGlobals, dir->flags, dir->folderID, kCalculatedCatalogRefNum);
+#if DEBUG_XATTR
+		printf ("%s: Record folderID=%d for prime modulus calculations\n", __FUNCTION__, dir->folderID); 
+#endif
+	}
+#if 0 //DEBUG_XATTR
+	else {
+		if (result == 0) 
+			printf ("%s: Ignoring folder ID %d from prime modulus calculuation\n", __FUNCTION__, dir->folderID); 
+	}
+#endif 
+	result = 0;	/* ignore result from record xattr */
+
 	if (dirID < kHFSFirstUserCatalogNodeID  &&
             dirID != kHFSRootFolderID) {
 		RcdError(gScavGlobals, E_InvalidID);
@@ -342,7 +403,7 @@ CheckDirectory(const HFSPlusCatalogKey * key, const HFSPlusCatalogFolder * dir)
 	if (dirID >= gCIS.nextCNID )
 		gCIS.nextCNID = dirID + 1;
 
-	gCIS.encodings |= (1 << MapEncodingToIndex(dir->textEncoding & 0x7F));
+	gCIS.encodings |= (u_int64_t)(1ULL << MapEncodingToIndex(dir->textEncoding & 0x7F));
 
 	CheckBSDInfo(key, &dir->bsdInfo, true);
 	
@@ -373,17 +434,28 @@ CheckFile(const HFSPlusCatalogKey * key, const HFSPlusCatalogFile * file)
 				filename, &len);
 	filename[len] = '\0';
 
-	/* Check reserved fields
-	 *
-	 * NOTE: the bit 7 (mask 0x80) of the flags byte isn't used by HFS or HFS Plus.
-	 * It was used by MFS to indicate that a file record was in use.  However, Inside
-	 * Macintosh: Files documents this bit for HFS volumes, and some non-Mac implementations
-	 * appear to set the bit.  Therefore, we ignore it.
+	/* 3843766: Check for reserved bits removed to support new bits in future */
+
+	/* 3971173, 3977448 : HFS does not clear the attribute bit after it deletes the 
+	 * last attribute.  Fix it later in 3977448 by removing the call for checking attrs
 	 */
-	if ((file->flags & (UInt16) ~(0X83)) != 0) {
-		RcdError(gScavGlobals, E_CatalogFlagsNotZero);
-		gScavGlobals->CBTStat |= S_ReservedNotZero;
+	result = CheckIfAttributeExists(file->fileID);
+	if (result == 1) {
+		/* Atleast one extended attribute exists for this file ID */
+		/* 3857929: Record if file record has attribute and/or security bit set */
+		RecordXAttrBits(gScavGlobals, file->flags, file->fileID, kCalculatedCatalogRefNum);
+#if DEBUG_XATTR
+		printf ("%s: Record fileID=%d for prime modulus calculations\n", __FUNCTION__, file->fileID); 
+#endif
 	}
+#if 0 //DEBUG_XATTR
+	else { 
+		if (result == 0) 
+			printf ("%s: Ignoring file ID %d from prime modulus calculuation\n", __FUNCTION__, file->fileID);
+	}
+#endif 
+	result = 0;	/* ignore result from record xattr */
+
 	fileID = file->fileID;
 	if (fileID < kHFSFirstUserCatalogNodeID) {
 		RcdError(gScavGlobals, E_InvalidID);
@@ -393,7 +465,7 @@ CheckFile(const HFSPlusCatalogKey * key, const HFSPlusCatalogFile * file)
 	if (fileID >= gCIS.nextCNID )
 		gCIS.nextCNID = fileID + 1;
 
-	gCIS.encodings |= (1 << MapEncodingToIndex(file->textEncoding & 0x7F));
+	gCIS.encodings |= (u_int64_t)(1ULL << MapEncodingToIndex(file->textEncoding & 0x7F));
 
 	CheckBSDInfo(key, &file->bsdInfo, false);
 
@@ -505,7 +577,7 @@ CheckDirectory_HFS(const HFSCatalogKey * key, const HFSCatalogFolder * dir)
 
 	dirID = dir->folderID;
 
-	if (dir->flags != 0) {
+	if ((dir->flags & (kHFSFileLockedMask | kHFSThreadExistsMask)) != 0) {
 		RcdError(gScavGlobals, E_CatalogFlagsNotZero);
 		gScavGlobals->CBTStat |= S_ReservedNotZero;
 	}
@@ -541,16 +613,8 @@ CheckFile_HFS(const HFSCatalogKey * key, const HFSCatalogFile * file)
 	if (file->flags & kHFSThreadExistsMask)
 		++gCIS.filesWithThreads;
 
-	/* Check reserved fields
-	 *
-	 * NOTE: the bit 7 (mask 0x80) of the flags byte isn't used
-	 * by HFS It was used by MFS to indicate that a file record
-	 * was in use.  However, Inside Macintosh: Files documents
-	 * this bit for HFS volumes, and some non-Mac implementations
-	 * appear to set the bit.  Therefore, we ignore it.
-	 */
-	if ((file->flags & (UInt8) ~(0X83)) ||
-	    (file->dataStartBlock)          ||
+	/* 3843766: Check for reserved bits removed to support new bits in future */
+	if ((file->dataStartBlock)          ||
 	    (file->rsrcStartBlock)          ||
 	    (file->reserved))
 	{
@@ -810,23 +874,12 @@ CheckCatalogName(u_int16_t charCount, const u_int16_t *uniChars, u_int32_t paren
     // look for Unicode decomposition errors in file system object names created before Jaguar (10.2)
     if ( FixDecomps( charCount, uniChars, &newName.ustr ) )
     {
-        UInt16				recSize;
-        CatalogKey			catKey;
-        CatalogRecord		record;
-       
         PrintError( gScavGlobals, E_IllegalName, 0 );
         if ( gScavGlobals->logLevel >= kDebugLog ) {
             printf( "\tillegal name is 0x" );
             PrintName( charCount, (UInt8 *) uniChars, true );
         }
 
-        // make sure new name isn't already there
-        BuildCatalogKey( parentID, &newName, true, &catKey );
-        result = SearchBTreeRecord( gScavGlobals->calculatedCatalogFCB, &catKey, kNoHint, NULL, 
-                                    &record, &recSize, NULL );
-        if ( result == noErr )
-            return( noErr );
-
         // we will copy the old and new names to our RepairOrder.  The names will
         // look like this:
         // 	 2 byte length of old name
@@ -1415,15 +1468,4 @@ FixDecomps(	u_int16_t charCount, const u_int16_t *inFilename, HFSUniStr255 *outF
 } /* FixDecomps */
 
 
-static void PrintName( int theCount, const UInt8 *theNamePtr, Boolean isUnicodeString )
-{
-    int			myCount;
- 	int			i;
-    
-    myCount = (isUnicodeString) ? (theCount * 2) : theCount;
-    for ( i = 0; i < myCount; i++ ) 
-        printf( "%02X ", *(theNamePtr + i) );
-    printf( "\n" );
-
-} /* PrintName */
 
