@@ -3,22 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
+ * Reserved.  This file contains Original Code and/or Modifications of
+ * Original Code as defined in and that are subject to the Apple Public
+ * Source License Version 1.0 (the 'License').  You may not use this file
+ * except in compliance with the License.  Please obtain a copy of the
+ * License at http://www.apple.com/publicsource and read it before using
+ * this file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License."
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -127,10 +126,12 @@ DoAgain:
 	    && scanCount == 0
 	    && checkLevel != kForceCheck
 	    && !(checkLevel == kPartialCheck && repairLevel == kForceRepairs)) {
-	    printf("fsck_hfs: Volume is journaled.  No checking performed.\n");
-	    printf("fsck_hfs: Use the -f option to force checking.\n");
-	    scavError = 0;
-	    goto termScav;
+		if (!guiControl) {
+	    		printf("fsck_hfs: Volume is journaled.  No checking performed.\n");
+	    		printf("fsck_hfs: Use the -f option to force checking.\n");
+		}
+		scavError = 0;
+		goto termScav;
 	}
 	dataArea.calculatedVCB->vcbDriveNumber = fsReadRef;
 	dataArea.calculatedVCB->vcbDriverWriteRef = fsWriteRef;
@@ -302,8 +303,17 @@ void ScavCtrl( SGlobPtr GPtr, UInt32 ScavOp, short *ScavRes )
 				if (GPtr->chkLevel == kNeverCheck) {
 					if (clean == -1)
 						result = R_BadSig;
-					else if (clean == 0)
-						result = R_Dirty;
+					else if (clean == 0) {
+						/*
+						 * We lie for journaled file systems since
+						 * they get cleaned up in mount by replaying
+						 * the journal.
+						 */
+						if (CheckIfJournaled(GPtr))
+							GPtr->cleanUnmount = true;
+						else
+							result = R_Dirty;
+					}
 					break;
 				}
 
@@ -862,6 +872,11 @@ static int ScavSetUp( SGlob *GPtr)
 		}
 		return( R_NoMem );
 	}
+	
+	// Convert the security attribute name from utf8 to utf16.  This will
+	// avoid repeated conversion of all extended attributes to compare with
+	// security attribute name
+	(void) utf_decodestr(KAUTH_FILESEC_XATTR, strlen(KAUTH_FILESEC_XATTR), GPtr->securityAttrName, &GPtr->securityAttrLen);
 
 	return( noErr );
 
