@@ -42,6 +42,7 @@ enum {
 	kBMS_PoolMax		= 2000
 };
 
+
 #define kAllBitsSetInWord	0xFFFFFFFFul
 
 enum {
@@ -104,9 +105,13 @@ static void       BMS_MaxDepth(BMS_Node * root, int depth, int *maxdepth);
 int
 BitMapCheckBegin(SGlobPtr g)
 {
+	Boolean				isHFSPlus;
+
 	if (gBitMapInited)
 		return (0);
 
+	isHFSPlus = VolumeObjectIsHFSPlus( );
+
 	gFullBitmapSegment = (UInt32 *)malloc(kBytesPerSegment);
 	memset((void *)gFullBitmapSegment, 0xff, kBytesPerSegment);
 
@@ -125,7 +130,7 @@ BitMapCheckBegin(SGlobPtr g)
 	gBitMapInited = 1;
 	gBitsMarked = 0;
 
-	if (g->isHFSPlus) {
+	if (isHFSPlus) {
 		UInt16	alignBits;
 		
 		/*
@@ -347,13 +352,13 @@ CaptureBitmapBits(UInt32 startBit, UInt32 bitCount)
 			bitMask &= ~(kAllBitsSetInWord >> (firstBit + numBits)); // turn off bits after last
 		}
 
-		if (*currentWord & bitMask) {
+		if (SWAP_BE32(*currentWord) & bitMask) {
 			overlap = true;
 
 		//	printf("(1) overlapping file blocks! word: 0x%08x, mask: 0x%08x\n", *currentWord, bitMask);
 		}
 		
-		*currentWord |= bitMask;  /* set the bits in the bitmap */
+		*currentWord |= SWAP_BE32(bitMask);  /* set the bits in the bitmap */
 		
 		bitCount -= numBits;
 		++currentWord;
@@ -379,13 +384,13 @@ CaptureBitmapBits(UInt32 startBit, UInt32 bitCount)
 			wordsLeft = kWordsPerSegment;
 		}
 		
-		if (*currentWord & bitMask) {
+		if (SWAP_BE32(*currentWord) & bitMask) {
 			overlap = true;
 
 		//	printf("(2) overlapping file blocks! word: 0x%08x, mask: 0x%08x\n", *currentWord, bitMask);
 		}
 		
-		*currentWord |= bitMask;  /* set the bits in the bitmap */
+		*currentWord |= SWAP_BE32(bitMask);  /* set the bits in the bitmap */
 
 		bitCount -= kBitsPerWord;
 		++currentWord;
@@ -409,13 +414,13 @@ CaptureBitmapBits(UInt32 startBit, UInt32 bitCount)
 			wordsLeft = kWordsPerSegment;
 		}
 		
-		if (*currentWord & bitMask) {
+		if (SWAP_BE32(*currentWord) & bitMask) {
 			overlap = true;
 
 		//	printf("(3) overlapping file blocks! word: 0x%08x, mask: 0x%08x\n", *currentWord, bitMask);
 		}
 		
-		*currentWord |= bitMask;  /* set the bits in the bitmap */
+		*currentWord |= SWAP_BE32(bitMask);  /* set the bits in the bitmap */
 
 		TestSegmentBitmap(startBit);
 	}
@@ -480,13 +485,13 @@ ReleaseBitmapBits(UInt32 startBit, UInt32 bitCount)
 			bitMask &= ~(kAllBitsSetInWord >> (firstBit + numBits)); // turn off bits after last
 		}
 
-		if ((*currentWord & bitMask) != bitMask) {
+		if ((SWAP_BE32(*currentWord) & bitMask) != bitMask) {
 			overlap = true;
 
 		//	printf("(1) overlapping file blocks! word: 0x%08x, mask: 0x%08x\n", *currentWord, bitMask);
 		}
 		
-		*currentWord &= ~bitMask;  /* clear the bits in the bitmap */
+		*currentWord &= SWAP_BE32(~bitMask);  /* clear the bits in the bitmap */
 		
 		bitCount -= numBits;
 		++currentWord;
@@ -512,13 +517,13 @@ ReleaseBitmapBits(UInt32 startBit, UInt32 bitCount)
 			wordsLeft = kWordsPerSegment;
 		}
 		
-		if ((*currentWord & bitMask) != bitMask) {
+		if ((SWAP_BE32(*currentWord) & bitMask) != bitMask) {
 			overlap = true;
 
 		//	printf("(2) overlapping file blocks! word: 0x%08x, mask: 0x%08x\n", *currentWord, bitMask);
 		}
 		
-		*currentWord &= ~bitMask;  /* clear the bits in the bitmap */
+		*currentWord &= SWAP_BE32(~bitMask);  /* clear the bits in the bitmap */
 
 		bitCount -= kBitsPerWord;
 		++currentWord;
@@ -542,13 +547,13 @@ ReleaseBitmapBits(UInt32 startBit, UInt32 bitCount)
 			wordsLeft = kWordsPerSegment;
 		}
 		
-		if ((*currentWord & bitMask) != bitMask) {
+		if ((SWAP_BE32(*currentWord) & bitMask) != bitMask) {
 			overlap = true;
 
 		//	printf("(3) overlapping file blocks! word: 0x%08x, mask: 0x%08x\n", *currentWord, bitMask);
 		}
 		
-		*currentWord &= ~bitMask;  /* set the bits in the bitmap */
+		*currentWord &= SWAP_BE32(~bitMask);  /* set the bits in the bitmap */
 
 		TestSegmentBitmap(startBit);
 	}
@@ -574,10 +579,12 @@ CheckVolumeBitMap(SGlobPtr g, Boolean repair)
 	ReleaseBlockOptions relOpt;
 	SFCB * fcb;
 	SVCB * vcb;
+	Boolean	 isHFSPlus;
 	int err = 0;
 	
 	vcb = g->calculatedVCB;
 	fcb = g->calculatedAllocationsFCB;
+	isHFSPlus = VolumeObjectIsHFSPlus( );
 	
 	if ( vcb->vcbFreeBlocks != (vcb->vcbTotalBlocks - gBitsMarked) ) {
 		vcb->vcbFreeBlocks = vcb->vcbTotalBlocks - gBitsMarked;
@@ -587,8 +594,11 @@ CheckVolumeBitMap(SGlobPtr g, Boolean repair)
 	vbmBlockP = (UInt8 *)NULL;
 	block.buffer = (void *)NULL;
 	relOpt = kReleaseBlock;
-	bitsWithinFileBlkMask = (fcb->fcbBlockSize * 8) - 1;
-	fileBlk = (g->isHFSPlus ? 0 : vcb->vcbVBMSt);
+	if ( isHFSPlus )
+		bitsWithinFileBlkMask = (fcb->fcbBlockSize * 8) - 1;
+	else
+		bitsWithinFileBlkMask = (kHFSBlockSize * 8) - 1;
+	fileBlk = (isHFSPlus ? 0 : vcb->vcbVBMSt);
 
 	/* 
 	 * Loop through all the bitmap segments and compare
@@ -601,7 +611,7 @@ CheckVolumeBitMap(SGlobPtr g, Boolean repair)
 		 * When we cross file block boundries read a new block from disk.
 		 */
 		if ((bit & bitsWithinFileBlkMask) == 0) {
-			if (g->isHFSPlus) {
+			if (isHFSPlus) {
 				if (block.buffer) {
 					err = ReleaseFileBlock(fcb, &block, relOpt);
 					ReturnIfError(err);
@@ -609,10 +619,10 @@ CheckVolumeBitMap(SGlobPtr g, Boolean repair)
 				err = GetFileBlock(fcb, fileBlk, kGetBlock, &block);
 			} else /* plain HFS */ {
 				if (block.buffer) {
-					err = ReleaseVolumeBlock(vcb, &block, relOpt);
+					err = ReleaseVolumeBlock(vcb, &block, relOpt | kSkipEndianSwap);
 					ReturnIfError(err);
 				}
-				err = GetVolumeBlock(vcb, fileBlk, kGetBlock, &block);
+				err = GetVolumeBlock(vcb, fileBlk, kGetBlock | kSkipEndianSwap, &block);
 			}
 			ReturnIfError(err);
 
@@ -621,10 +631,9 @@ CheckVolumeBitMap(SGlobPtr g, Boolean repair)
 			g->TarBlock = fileBlk;
 			++fileBlk;
 		}
-
 		if (memcmp(buffer, vbmBlockP + (bit & bitsWithinFileBlkMask)/8, kBytesPerSegment) == 0)
 			continue;
-			
+
 		if (repair) {
 			bcopy(buffer, vbmBlockP + (bit & bitsWithinFileBlkMask)/8, kBytesPerSegment);
 			relOpt = kForceWriteBlock;
@@ -655,10 +664,10 @@ CheckVolumeBitMap(SGlobPtr g, Boolean repair)
 	}
 
 	if (block.buffer) {
-		if (g->isHFSPlus)
+		if (isHFSPlus)
 			(void) ReleaseFileBlock(fcb, &block, relOpt);
 		else
-			(void) ReleaseVolumeBlock(vcb, &block, relOpt);
+			(void) ReleaseVolumeBlock(vcb, &block, relOpt | kSkipEndianSwap);
 	}
 
 	return (0);
