@@ -3,22 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
+ * Reserved.  This file contains Original Code and/or Modifications of
+ * Original Code as defined in and that are subject to the Apple Public
+ * Source License Version 1.0 (the 'License').  You may not use this file
+ * except in compliance with the License.  Please obtain a copy of the
+ * License at http://www.apple.com/publicsource and read it before using
+ * this file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License."
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -27,7 +26,7 @@
 #include <sys/param.h>
 #include <sys/mount.h>
 #include <sys/ioctl.h>
-#include <dev/disk.h>
+#include <sys/disk.h>
 
 #include <hfs/hfs_mount.h>
 
@@ -202,8 +201,7 @@ checkfilesys(char * filesys)
 		chkLev = kNeverCheck;
 		repLev = kNeverRepair;
 		logLev = kFatalLog;
-	}
-	if (force) {
+	} else if (force) {
 		chkLev = kForceCheck;
 	}
 	if (preen) {
@@ -346,7 +344,7 @@ setup( char *dev, int *blockDevice_fdPtr, int *canWritePtr )
 		printf("\n");
 
 	/* Initialize the cache */
-	ioctl(fsreadfd, DKIOCBLKSIZE, &devBlockSize);
+	ioctl(fsreadfd, DKIOCGETBLOCKSIZE, &devBlockSize);
 	if (CacheInit (&fscache, fsreadfd, fswritefd, devBlockSize,
 			CACHE_IOSIZE, CACHE_BLOCKS, CACHE_HASHSIZE) != EOK) {
 		pfatal("Can't initialize disk cache\n");
@@ -393,7 +391,7 @@ static void getWriteAccess( char *dev, int *blockDevice_fdPtr, int *canWritePtr
 	}
 	
 	// get count of mounts then get the info for each 
-	myMountsCount = getfsstat( NULL, 0, MNT_WAIT );
+	myMountsCount = getfsstat( NULL, 0, MNT_NOWAIT );
 	if ( myMountsCount < 0 )
 		goto ExitThisRoutine;
 
@@ -402,7 +400,7 @@ static void getWriteAccess( char *dev, int *blockDevice_fdPtr, int *canWritePtr
 		goto ExitThisRoutine;
 	myMountsCount = getfsstat( 	myPtr, 
 								(sizeof(struct statfs) * myMountsCount), 
-								MNT_WAIT );
+								MNT_NOWAIT );
 	if ( myMountsCount < 0 )
 		goto ExitThisRoutine;
 
