@@ -1,158 +1,63 @@
 /*
- * Copyright (c) 1999-2002 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1999-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.2 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
-/*
- * Copyright (c) 1997-2000 Apple Computer, Inc. All Rights Reserved
- *
- *		MODIFICATION HISTORY (most recent first):
-*	   28-Jan-2000	Don Brady		Add support for hfs encoding modules (radar #2428608).
- *	   26-Jan-1999	Don Brady		Synchronize volume/root create dates (radar #2299171).
- *	   24-Jun-1998	Don Brady		Pass time zone info to hfs on mount (radar #2226387).
- */
-#include <sys/cdefs.h>
-#include <sys/param.h>
-#include <sys/mount.h>
+
 #include <sys/types.h>
+
+#include <sys/attr.h>
+#include <sys/mount.h>
 #include <sys/stat.h>
+#include <sys/sysctl.h>
+#include <sys/time.h>
+#include <sys/uio.h>
 #include <sys/vnode.h>
 #include <sys/wait.h>
-#include <sys/time.h> // gettimeofday
 
 #include <ctype.h>
 #include <err.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <grp.h>
+#include <limits.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <fcntl.h>
+
 #include <hfs/hfs_mount.h>
 #include <hfs/hfs_format.h>
-#include <sys/attr.h>
-#include <errno.h>
 
 /* Sensible wrappers over the byte-swapping routines */
 #include "hfs_endian.h"
 
-/* bek 5/20/98 - [2238317] - mntopts.h needs to be installed in a public place */
+#include "../disklib/mntopts.h"
 
-#define Radar_2238317 1
 
-#if ! Radar_2238317
-
-#include <mntopts.h>
-
-#else //  Radar_2238317
-
-/*-
- * Copyright (c) 1994
- *      The Regents of the University of California.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)mntopts.h	8.7 (Berkeley) 3/29/95
- */
-
-struct mntopt {
-	const char *m_option;	/* option name */
-	int m_inverse;		/* if a negative option, eg "dev" */
-	int m_flag;		/* bit to set, eg. MNT_RDONLY */
-	int m_altloc;		/* 1 => set bit in altflags */
-};
-
-/* User-visible MNT_ flags. */
-#define MOPT_ASYNC		{ "async",	0, MNT_ASYNC, 0 }
-#define MOPT_NODEV		{ "dev",	1, MNT_NODEV, 0 }
-#define MOPT_NOEXEC		{ "exec",	1, MNT_NOEXEC, 0 }
-#define MOPT_NOSUID		{ "suid",	1, MNT_NOSUID, 0 }
-#define MOPT_RDONLY		{ "rdonly",	0, MNT_RDONLY, 0 }
-#define MOPT_SYNC		{ "sync",	0, MNT_SYNCHRONOUS, 0 }
-#define MOPT_UNION		{ "union",	0, MNT_UNION, 0 }
-#define MOPT_USERQUOTA		{ "userquota",	0, 0, 0 }
-#define MOPT_GROUPQUOTA		{ "groupquota",	0, 0, 0 }
+/* This really belongs in mntopts.h */
 #define MOPT_PERMISSIONS	{ "perm", 1, MNT_UNKNOWNPERMISSIONS, 0 }
 
-/* Control flags. */
-#define MOPT_FORCE		{ "force",	0, MNT_FORCE, 0 }
-#define MOPT_UPDATE		{ "update",	0, MNT_UPDATE, 0 }
-#define MOPT_RO			{ "ro",		0, MNT_RDONLY, 0 }
-#define MOPT_RW			{ "rw",		1, MNT_RDONLY, 0 }
-
-/* This is parsed by mount(8), but is ignored by specific mount_*(8)s. */
-#define MOPT_AUTO		{ "auto",	0, 0, 0 }
-
-#define MOPT_FSTAB_COMPAT						\
-	MOPT_RO,							\
-	MOPT_RW,							\
-	MOPT_AUTO
-
-/* Standard options which all mounts can understand. */
-#define MOPT_STDOPTS							\
-	MOPT_USERQUOTA,							\
-	MOPT_GROUPQUOTA,						\
-	MOPT_FSTAB_COMPAT,						\
-	MOPT_NODEV,							\
-	MOPT_NOEXEC,							\
-	MOPT_NOSUID,							\
-	MOPT_RDONLY,							\
-	MOPT_UNION,								\
-	MOPT_PERMISSIONS
-
-void getmntopts __P((const char *, const struct mntopt *, int *, int *));
-extern int getmnt_silent;
-
-#endif // Radar_2238317
-
 struct mntopt mopts[] = {
 	MOPT_STDOPTS,
+	MOPT_PERMISSIONS,
 	MOPT_UPDATE,
 	{ NULL }
 };
@@ -166,6 +71,10 @@ gid_t	a_gid __P((char *));
 uid_t	a_uid __P((char *));
 mode_t	a_mask __P((char *));
 struct hfs_mnt_encoding * a_encoding __P((char *));
+struct hfs_mnt_encoding * get_encoding_pref __P((char *));
+int	get_encoding_bias __P((void));
+unsigned int  get_default_encoding(void);
+
 void	usage __P((void));
 
 
@@ -347,7 +256,7 @@ void syncCreateDate(const char *mntpt, u_long localCreateTime)
  *
  * Note: unloading of encoding converter modules is done in the kernel
  */
-static void
+static int
 load_encoding(struct hfs_mnt_encoding *encp)
 {
 	int pid;
@@ -357,11 +266,14 @@ load_encoding(struct hfs_mnt_encoding *encp)
 	char kmodfile[MAXPATHLEN];
 	
 	/* MacRoman encoding (0) is built into the kernel */
-	if (encp->encoding_id == 0) return;
+	if (encp->encoding_id == 0)
+		return (0);
 
 	sprintf(kmodfile, "%sHFS_Mac%s.kext", ENCODING_MODULE_PATH, encp->encoding_name);
-	if (stat(kmodfile, &sb) == -1)
-		errx(1, "unable to find: %s", kmodfile);
+	if (stat(kmodfile, &sb) == -1) {
+		fprintf(stdout, "unable to find: %s\n", kmodfile);
+		return (-1);
+	}
 
 	loaded = 0;
 	pid = fork();
@@ -376,8 +288,11 @@ load_encoding(struct hfs_mnt_encoding *encp)
 		}
 	}
 
-	if (!loaded)
-		errx(1, "unable to load: %s", kmodfile);
+	if (!loaded) {
+		fprintf(stderr, "unable to load: %s\n", kmodfile);
+		return (-1);
+	}
+	return (0);
 }
 
 int
@@ -387,7 +302,7 @@ main(argc, argv)
 {
 	struct hfs_mount_args args;
 	int ch, mntflags;
-	char *dev, *dir;
+	char *dev, dir[MAXPATHLEN];
 	int mountStatus;
 	struct timeval dummy_timeval; /* gettimeofday() crashes if the first argument is NULL */
 	u_long localCreateTime;
@@ -495,7 +410,9 @@ main(argc, argv)
 	}
 
 	dev = argv[0];
-	dir = argv[1];
+
+	if (realpath(argv[1], dir) == NULL)
+		err(1, "realpath %s", dir);
 
 	args.fspec = dev;
 	args.export.ex_root = DEFAULT_ROOTUID;
@@ -510,7 +427,8 @@ main(argc, argv)
 
 	/* load requested encoding (if any) for hfs volume */
 	if (encp != NULL) {
-		load_encoding(encp);
+		if (load_encoding(encp) != 0)
+			exit(1);  /* load failure */
 		args.hfs_encoding = encp->encoding_id;
 	}
 	
@@ -523,9 +441,16 @@ main(argc, argv)
            	if (args.flags == VNOVAL)
             		args.flags = 0;
 
-		if (args.hfs_encoding == (u_long)VNOVAL) 
+		if ((args.hfs_encoding == (u_long)VNOVAL) && (encp == NULL)) {
 			args.hfs_encoding = 0;
 
+			/* Check if volume had a previous encoding preference. */
+			encp = get_encoding_pref(dev);
+			if (encp != NULL) {
+				if (load_encoding(encp) == 0)
+					args.hfs_encoding = encp->encoding_id;
+			}
+		}
 		/* when the mountpoint is root, use default values */
 		if (strcmp(dir, "/") == 0) {
 			sb.st_mode = 0777;
@@ -684,6 +609,109 @@ unknown:
 	return (NULL);
 }
 
+
+/*
+ * Get file system's encoding preference.
+ */
+struct hfs_mnt_encoding *
+get_encoding_pref(char *dev)
+{
+	char buffer[HFS_BLOCK_SIZE];
+	struct hfs_mnt_encoding *enclist;
+	HFSMasterDirectoryBlock * mdbp;
+	int encoding = -1;
+	int elements;
+	int fd;
+	int i;
+
+	/* Can only load encoding modules if root. */
+	if (geteuid() != 0)
+		return (NULL);
+
+	fd = open(dev, O_RDONLY | O_NDELAY, 0);
+	if (fd == -1)
+		return (NULL);
+
+     	if (pread(fd, buffer, sizeof(buffer), 1024) != sizeof(buffer)) {
+     		close(fd);
+		return (NULL);
+	}
+    	close(fd);
+
+	mdbp = (HFSMasterDirectoryBlock *) buffer;
+	if (SWAP_BE16(mdbp->drSigWord) != kHFSSigWord ||
+	    SWAP_BE16(mdbp->drEmbedSigWord) == kHFSPlusSigWord ) {
+		return (NULL);
+	}
+	encoding = GET_HFS_TEXT_ENCODING(SWAP_BE32(mdbp->drFndrInfo[4]));
+
+	if (encoding == -1) {
+		encoding = get_encoding_bias();
+		if (encoding == 0 || encoding == -1)
+			encoding = get_default_encoding();
+	}
+
+	elements = sizeof(hfs_mnt_encodinglist) / sizeof(struct hfs_mnt_encoding);
+	for (i=0, enclist = hfs_mnt_encodinglist; i < elements; i++, enclist++) {
+		if (enclist->encoding_id == encoding)
+			return (enclist);
+	}
+
+	return (NULL);
+}
+
+/*
+ * Get kernel's encoding bias.
+ */
+int
+get_encoding_bias()
+{
+        int mib[3];
+        size_t buflen = sizeof(int);
+        struct vfsconf vfc;
+        int hint = 0;
+
+        if (getvfsbyname("hfs", &vfc) < 0)
+		goto error;
+
+        mib[0] = CTL_VFS;
+        mib[1] = vfc.vfc_typenum;
+        mib[2] = HFS_ENCODINGBIAS;
+ 
+	if (sysctl(mib, 3, &hint, &buflen, NULL, 0) < 0)
+ 		goto error;
+	return (hint);
+error:
+	return (-1);
+}
+
+#define __kCFUserEncodingFileName ("/.CFUserTextEncoding")
+
+unsigned int
+get_default_encoding()
+{
+	struct passwd *passwdp;
+
+	if ((passwdp = getpwuid(0))) {	/* root account */
+		char buffer[MAXPATHLEN + 1];
+		int fd;
+
+		strcpy(buffer, passwdp->pw_dir);
+		strcat(buffer, __kCFUserEncodingFileName);
+
+		if ((fd = open(buffer, O_RDONLY, 0)) > 0) {
+			size_t readSize;
+
+			readSize = read(fd, buffer, MAXPATHLEN);
+			buffer[(readSize < 0 ? 0 : readSize)] = '\0';
+			close(fd);
+			return strtol(buffer, NULL, 0);
+		}
+	}
+	return (0);	/* Fallback to smRoman */
+}
+
+
 void
 usage()
 {
