@@ -1,24 +1,21 @@
 /*
- * Copyright (c) 1999-2002 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1999-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -31,26 +28,30 @@
 
 */
 
-#include <unistd.h>
-#include <err.h>
-#include <fcntl.h>
-
 #include <sys/param.h>
+#include <sys/types.h>
 #include <sys/time.h>
 #include <sys/errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/sysctl.h>
 
+#include <err.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <pwd.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <CoreFoundation/CFString.h>
+#include <CoreFoundation/CFStringEncodingExt.h>
+
 extern Boolean _CFStringGetFileSystemRepresentation(CFStringRef string, UInt8 *buffer, CFIndex maxBufLen);
 
 
 #include <hfs/hfs_format.h>
+#include <hfs/hfs_mount.h>
 #include "hfs_endian.h"
 
 #include "newfs_hfs.h"
@@ -132,6 +133,8 @@ static UInt32 DivideAndRoundUp __P((UInt32 numerator, UInt32 denominator));
 static int ConvertUTF8toUnicode __P((const UInt8* source, UInt32 bufsize,
 		UniChar* unibuf, UInt16 *charcount));
 
+static int getencodinghint(char *name);
+
 
 void SETOFFSET (void *buffer, UInt16 btNodeSize, SInt16 recOffset, SInt16 vecOffset);
 #define SETOFFSET(buf,ndsiz,offset,rec)		\
@@ -141,6 +144,11 @@ void SETOFFSET (void *buffer, UInt16 btNodeSize, SInt16 recOffset, SInt16 vecOff
 
 #define ROUNDUP(x, u)	(((x) % (u) == 0) ? (x) : ((x)/(u) + 1) * (u))
 
+#define ENCODING_TO_BIT(e)                               \
+          ((e) < 48 ? (e) :                              \
+          ((e) == kCFStringEncodingMacUkrainian ? 48 :   \
+          ((e) == kCFStringEncodingMacFarsi ? 49 : 0)))
+
 /*
  * make_hfs
  *	
@@ -179,6 +187,8 @@ make_hfs(const DriveInfo *driveInfo,
 	if (nodeBuffer == NULL || mdbp == NULL) 
 		err(1, NULL);
 
+	defaults->encodingHint = getencodinghint(defaults->volumeName);
+
 	/* MDB Initialized in native byte order */
 	InitMDB(defaults, driveInfo->totalSectors, mdbp);
 
@@ -229,11 +239,10 @@ make_hfs(const DriveInfo *driveInfo,
 
 		(UInt16)mdbp->drFreeBks -= gDTDBFork.blockCount +
 				   	gReadMeFork.blockCount +
-				   	gSystemFork.blockCount +
-					defaults->hfsWrapperFreeBlks;
+				   	gSystemFork.blockCount;
 		mdbp->drEmbedExtent.startBlock = mdbp->drNmAlBlks - (UInt16)mdbp->drFreeBks;
 		mdbp->drEmbedExtent.blockCount = (UInt16)mdbp->drFreeBks;
-		(UInt16)mdbp->drFreeBks = defaults->hfsWrapperFreeBlks;
+		(UInt16)mdbp->drFreeBks = 0;
 	}
 
 
@@ -311,6 +320,8 @@ make_hfsplus(const DriveInfo *driveInfo, hfsparams_t *defaults)
 	if (header == NULL)
 		err(1, NULL);
 
+	defaults->encodingHint = getencodinghint(defaults->volumeName);
+
 	/* VH Initialized in native byte order */
 	InitVH(defaults, driveInfo->totalSectors, header);
 
@@ -515,12 +526,13 @@ InitMDB(hfsparams_t *defaults, UInt32 driveBlocks, HFS_MDB *mdbp)
 		/* Find out what Mac encoding to use: */
 		maxchars = MIN(sizeof(unibuf)/sizeof(UniChar), CFStringGetLength(cfstr));
 		CFStringGetCharacters(cfstr, CFRangeMake(0, maxchars), unibuf);
-		cfOK = CFStringGetPascalString(cfstr, mdbp->drVN, sizeof(mdbp->drVN), GetDefaultEncoding());
+		cfOK = CFStringGetPascalString(cfstr, mdbp->drVN, sizeof(mdbp->drVN), defaults->encodingHint);
 		CFRelease(cfstr);
 
 		if (!cfOK) {
 			mdbp->drVN[0] = strlen(kDefaultVolumeNameStr);
 			bcopy(kDefaultVolumeNameStr, &mdbp->drVN[1], mdbp->drVN[0]);
+			defaults->encodingHint = 0;
 			warnx("invalid HFS name: \"%s\", using \"%s\" instead",
 			      defaults->volumeName, kDefaultVolumeNameStr);
 		}
@@ -528,7 +540,9 @@ InitMDB(hfsparams_t *defaults, UInt32 driveBlocks, HFS_MDB *mdbp)
 		bcopy(&mdbp->drVN[1], defaults->volumeName, mdbp->drVN[0]);
 		defaults->volumeName[mdbp->drVN[0]] = '\0';
 	}
-	
+	/* Save the encoding hint in the Finder Info (field 4). */
+	mdbp->drFndrInfo[4] = SET_HFS_TEXT_ENCODING(defaults->encodingHint);
+
 	mdbp->drWrCnt = kWriteSeqNum;
 
 	mdbp->drXTFlSize = mdbp->drXTClpSiz = defaults->extentsClumpSize;
@@ -613,8 +627,13 @@ InitVH(hfsparams_t *defaults, UInt64 sectors, HFSPlusVolumeHeader *hp)
 	/* note: add 2 for the Alternate VH, and VH */
 	blocksUsed = 2 + burnedBlocksBeforeVH + burnedBlocksAfterAltVH + bitmapBlocks;
 
-	hp->signature = kHFSPlusSigWord;
-	hp->version = kHFSPlusVersion;
+	if (defaults->flags & kMakeCaseSensitive) {
+		hp->signature = kHFSXSigWord;
+		hp->version = kHFSXVersion;
+	} else {
+		hp->signature = kHFSPlusSigWord;
+		hp->version = kHFSPlusVersion;
+	}
 	hp->attributes = kHFSVolumeUnmountedMask;
 	hp->lastMountedVersion = kHFSPlusMountVersion;
 
@@ -634,7 +653,7 @@ InitVH(hfsparams_t *defaults, UInt64 sectors, HFSPlusVolumeHeader *hp)
 	hp->rsrcClumpSize = defaults->rsrcClumpSize;
 	hp->dataClumpSize = defaults->dataClumpSize;
 	hp->nextCatalogID = defaults->nextFreeFileID;
-	hp->encodingsBitmap = 1;	/* just set to MacRoman */
+	hp->encodingsBitmap = 1 | (1 << ENCODING_TO_BIT(defaults->encodingHint));
 
 	/* set up allocation bitmap file */
 	hp->allocationFile.clumpSize = defaults->allocationClumpSize;
@@ -647,7 +666,6 @@ InitVH(hfsparams_t *defaults, UInt64 sectors, HFSPlusVolumeHeader *hp)
 	if (defaults->journaledHFS) {
 		hp->fileCount           = 2;
 		hp->attributes         |= kHFSVolumeJournaledMask;
-		hp->lastMountedVersion  = kHFSJMountVersion;
 		hp->nextCatalogID      += 2;
 
 		/*
@@ -779,21 +797,21 @@ WriteExtentsFile(const DriveInfo *driveInfo, UInt32 startingSector,
 	bthp->freeNodes		= SWAP_BE32 (SWAP_BE32 (bthp->totalNodes) - 1);  /* header */
 	bthp->clumpSize		= SWAP_BE32 (fileSize);
 
-	if (dp->signature == kHFSPlusSigWord) {
-		bthp->attributes	|= SWAP_BE32 (kBTBigKeysMask);
-		bthp->maxKeyLength	=  SWAP_BE16 (kHFSPlusExtentKeyMaximumLength);
-	} else {
-		bthp->maxKeyLength	= SWAP_BE16 (kHFSExtentKeyMaximumLength);
+	if (dp->flags & kMakeStandardHFS) {
+		bthp->maxKeyLength = SWAP_BE16 (kHFSExtentKeyMaximumLength);
 
 		/* wrapper has a bad-block extent record */
 		if (wrapper) {
-			bthp->treeDepth		= SWAP_BE16 (SWAP_BE16 (bthp->treeDepth) + 1);
-			bthp->leafRecords	= SWAP_BE32 (SWAP_BE32 (bthp->leafRecords) + 1);
-			bthp->rootNode		= SWAP_BE32 (1);
-			bthp->firstLeafNode	= SWAP_BE32 (1);
-			bthp->lastLeafNode	= SWAP_BE32 (1);
-			bthp->freeNodes		= SWAP_BE32 (SWAP_BE32 (bthp->freeNodes) - 1);
+			bthp->treeDepth     = SWAP_BE16 (SWAP_BE16 (bthp->treeDepth) + 1);
+			bthp->leafRecords   = SWAP_BE32 (SWAP_BE32 (bthp->leafRecords) + 1);
+			bthp->rootNode      = SWAP_BE32 (1);
+			bthp->firstLeafNode = SWAP_BE32 (1);
+			bthp->lastLeafNode  = SWAP_BE32 (1);
+			bthp->freeNodes     = SWAP_BE32 (SWAP_BE32 (bthp->freeNodes) - 1);
 		}
+	} else {
+		bthp->attributes |= SWAP_BE32 (kBTBigKeysMask);
+		bthp->maxKeyLength = SWAP_BE16 (kHFSPlusExtentKeyMaximumLength);
 	}
 	offset += sizeof(BTHeaderRec);
 
@@ -904,14 +922,14 @@ WriteJournalInfo(const DriveInfo *driveInfo, UInt32 startingSector,
     jibp->size    = dp->journalSize;
 
     jibp->flags  = SWAP_BE32(jibp->flags);
-    jibp->offset = SWAP_BE32(jibp->offset);
-    jibp->size   = SWAP_BE32(jibp->size);
+    jibp->offset = SWAP_BE64(jibp->offset);
+    jibp->size   = SWAP_BE64(jibp->size);
     
     WriteBuffer(driveInfo, startingSector, header->blockSize, buffer);
 
     jibp->flags  = SWAP_BE32(jibp->flags);
-    jibp->offset = SWAP_BE32(jibp->offset);
-    jibp->size   = SWAP_BE32(jibp->size);
+    jibp->offset = SWAP_BE64(jibp->offset);
+    jibp->size   = SWAP_BE64(jibp->size);
 }
 
 
@@ -966,10 +984,7 @@ WriteCatalogFile(const DriveInfo *driveInfo, UInt32 startingSector,
 	bthp->freeNodes		= SWAP_BE32 (SWAP_BE32 (bthp->totalNodes) - 2);  /* header and root */
 	bthp->clumpSize		= SWAP_BE32 (fileSize);
 
-	if (dp->signature == kHFSPlusSigWord) {
-		bthp->attributes	|= SWAP_BE32 (kBTVariableIndexKeysMask + kBTBigKeysMask);
-		bthp->maxKeyLength	=  SWAP_BE16 (kHFSPlusCatalogKeyMaximumLength);
-	} else {
+	if (dp->flags & kMakeStandardHFS) {
 		bthp->maxKeyLength	=  SWAP_BE16 (kHFSCatalogKeyMaximumLength);
 
 		if (dp->flags & kMakeHFSWrapper) {
@@ -979,6 +994,13 @@ WriteCatalogFile(const DriveInfo *driveInfo, UInt32 startingSector,
 			bthp->lastLeafNode	= SWAP_BE32 (SWAP_BE32 (bthp->firstLeafNode) + 1);
 			bthp->freeNodes		= SWAP_BE32 (SWAP_BE32 (bthp->freeNodes) - 2);  /* tree now split with 2 leaf nodes */
 		}
+	} else /* HFS+ */ {
+		bthp->attributes	|= SWAP_BE32 (kBTVariableIndexKeysMask + kBTBigKeysMask);
+		bthp->maxKeyLength	=  SWAP_BE16 (kHFSPlusCatalogKeyMaximumLength);
+		if (dp->flags & kMakeCaseSensitive)
+			bthp->keyCompareType = kHFSBinaryCompare;
+		else
+			bthp->keyCompareType = kHFSCaseFolding;
 	}
 	offset += sizeof(BTHeaderRec);
 
@@ -1022,7 +1044,7 @@ WriteCatalogFile(const DriveInfo *driveInfo, UInt32 startingSector,
 
 	SETOFFSET(buffer, nodeSize, offset, 4);
 
-	if (dp->signature == kHFSPlusSigWord) {
+	if ((dp->flags & kMakeStandardHFS) == 0) {
 		InitCatalogRoot_HFSPlus(dp, header, buffer + nodeSize);
 
 	} else if (wrapper) {
@@ -1068,7 +1090,6 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 	ndp->height = 1;
 	ndp->numRecords = SWAP_BE16 (dp->journaledHFS ? 6 : 2);
 	offset = sizeof(BTNodeDescriptor);
-
 	SETOFFSET(buffer, nodeSize, offset, ++index);
 
 	/*
@@ -1106,9 +1127,13 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 	cdp->folderID		= SWAP_BE32 (kHFSRootFolderID);
 	cdp->createDate		= SWAP_BE32 (dp->createDate);
 	cdp->contentModDate	= SWAP_BE32 (dp->createDate);
-	cdp->textEncoding	= SWAP_BE32 (GetDefaultEncoding());
+	cdp->textEncoding	= SWAP_BE32 (dp->encodingHint);
+	if (dp->flags & kUseAccessPerms) {
+		cdp->bsdInfo.ownerID  = SWAP_BE32 (dp->owner);
+		cdp->bsdInfo.groupID  = SWAP_BE32 (dp->group);
+		cdp->bsdInfo.fileMode = SWAP_BE16 (dp->mask | S_IFDIR);
+	}
 	offset += sizeof(HFSPlusCatalogFolder);
-
 	SETOFFSET(buffer, nodeSize, offset, ++index);
 
 	/*
@@ -1135,19 +1160,21 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 	 */
 	if (dp->journaledHFS) {
 		struct HFSUniStr255 *nodename1, *nodename2;
+		int uBytes1, uBytes2;
 
 		/* File record #1 */
 		ckp = (HFSPlusCatalogKey *)((UInt8 *)buffer + offset);
 		(void) ConvertUTF8toUnicode(HFS_JOURNAL_FILE, sizeof(ckp->nodeName.unicode),
 		                            ckp->nodeName.unicode, &ckp->nodeName.length);
 		ckp->nodeName.length = SWAP_BE16 (ckp->nodeName.length);
-		unicodeBytes = sizeof(UniChar) * SWAP_BE16 (ckp->nodeName.length);
-		ckp->keyLength = SWAP_BE16 (kHFSPlusCatalogKeyMinimumLength + unicodeBytes);
+		uBytes1 = sizeof(UniChar) * SWAP_BE16 (ckp->nodeName.length);
+		ckp->keyLength = SWAP_BE16 (kHFSPlusCatalogKeyMinimumLength + uBytes1);
 		ckp->parentID  = SWAP_BE32 (kHFSRootFolderID);
 		offset += SWAP_BE16 (ckp->keyLength) + 2;
 	
 		cfp = (HFSPlusCatalogFile *)((UInt8 *)buffer + offset);
 		cfp->recordType     = SWAP_BE16 (kHFSPlusFileRecord);
+		cfp->flags          = SWAP_BE16 (kHFSThreadExistsMask);
 		cfp->fileID         = SWAP_BE32 (dp->nextFreeFileID);
 		cfp->createDate     = SWAP_BE32 (dp->createDate + 1);
 		cfp->contentModDate = SWAP_BE32 (dp->createDate + 1);
@@ -1173,13 +1200,14 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 		(void) ConvertUTF8toUnicode(HFS_JOURNAL_INFO, sizeof(ckp->nodeName.unicode),
 		                            ckp->nodeName.unicode, &ckp->nodeName.length);
 		ckp->nodeName.length = SWAP_BE16 (ckp->nodeName.length);
-		unicodeBytes = sizeof(UniChar) * SWAP_BE16 (ckp->nodeName.length);
-		ckp->keyLength = SWAP_BE16 (kHFSPlusCatalogKeyMinimumLength + unicodeBytes);
+		uBytes2 = sizeof(UniChar) * SWAP_BE16 (ckp->nodeName.length);
+		ckp->keyLength = SWAP_BE16 (kHFSPlusCatalogKeyMinimumLength + uBytes2);
 		ckp->parentID  = SWAP_BE32 (kHFSRootFolderID);
 		offset += SWAP_BE16 (ckp->keyLength) + 2;
 	
 		cfp = (HFSPlusCatalogFile *)((UInt8 *)buffer + offset);
 		cfp->recordType     = SWAP_BE16 (kHFSPlusFileRecord);
+		cfp->flags          = SWAP_BE16 (kHFSThreadExistsMask);
 		cfp->fileID         = SWAP_BE32 (dp->nextFreeFileID + 1);
 		cfp->createDate     = SWAP_BE32 (dp->createDate);
 		cfp->contentModDate = SWAP_BE32 (dp->createDate);
@@ -1191,7 +1219,7 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 		cfp->userInfo.fdCreator	  = SWAP_BE32 (kHFSPlusCreator);
 		cfp->userInfo.fdFlags     = SWAP_BE16 (kIsInvisible + kNameLocked);
 		cfp->dataFork.logicalSize = SWAP_BE64(dp->blockSize);;
-		cfp->dataFork.totalBlocks = 1;
+		cfp->dataFork.totalBlocks = SWAP_BE32(1);
 
 		cfp->dataFork.extents[0].startBlock = SWAP_BE32 (header->journalInfoBlock);
 		cfp->dataFork.extents[0].blockCount = cfp->dataFork.totalBlocks;
@@ -1210,9 +1238,9 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 		ctp = (HFSPlusCatalogThread *)((UInt8 *)buffer + offset);
 		ctp->recordType = SWAP_BE16 (kHFSPlusFileThreadRecord);
 		ctp->parentID   = SWAP_BE32 (kHFSRootFolderID);
-		bcopy(nodename1, &ctp->nodeName, sizeof(UInt16) + unicodeBytes);
+		bcopy(nodename1, &ctp->nodeName, sizeof(UInt16) + uBytes1);
 		offset += (sizeof(HFSPlusCatalogThread)
-				- (sizeof(ctp->nodeName.unicode) - unicodeBytes) );
+				- (sizeof(ctp->nodeName.unicode) - uBytes1) );
 		SETOFFSET(buffer, nodeSize, offset, ++index);
 
 		/* Thread record for file #2 */
@@ -1225,9 +1253,9 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 		ctp = (HFSPlusCatalogThread *)((UInt8 *)buffer + offset);
 		ctp->recordType = SWAP_BE16 (kHFSPlusFileThreadRecord);
 		ctp->parentID   = SWAP_BE32 (kHFSRootFolderID);
-		bcopy(nodename2, &ctp->nodeName, sizeof(UInt16) + unicodeBytes);
+		bcopy(nodename2, &ctp->nodeName, sizeof(UInt16) + uBytes2);
 		offset += (sizeof(HFSPlusCatalogThread)
-				- (sizeof(ctp->nodeName.unicode) - unicodeBytes) );
+				- (sizeof(ctp->nodeName.unicode) - uBytes2) );
 		SETOFFSET(buffer, nodeSize, offset, ++index);
 	}
 }
@@ -1419,7 +1447,6 @@ InitSecondCatalogLeaf(const hfsparams_t *dp, void * buffer)
 	cfp->recordType			= SWAP_BE16 (kHFSFileRecord);
 	cfp->userInfo.fdType	= SWAP_BE32 (kReadMe_Type);
 	cfp->userInfo.fdCreator	= SWAP_BE32 (kReadMe_Creator);
-	cfp->userInfo.fdFlags	= SWAP_BE16 (kIsInvisible);
 	cfp->fileID				= SWAP_BE32 (kReadMe_FileID);
 	cfp->createDate			= SWAP_BE32 (timeStamp);
 	cfp->modifyDate			= SWAP_BE32 (timeStamp);
@@ -1946,3 +1973,30 @@ ConvertUTF8toUnicode(const UInt8* source, UInt32 bufsize, UniChar* unibuf,
 
 	return (0);
 }
+
+/*
+ * Derive the encoding hint for the given name.
+ */
+static int
+getencodinghint(char *name)
+{
+        int mib[3];
+        size_t buflen = sizeof(int);
+        struct vfsconf vfc;
+        int hint = 0;
+
+        if (getvfsbyname("hfs", &vfc) < 0)
+		goto error;
+
+        mib[0] = CTL_VFS;
+        mib[1] = vfc.vfc_typenum;
+        mib[2] = HFS_ENCODINGHINT;
+ 
+	if (sysctl(mib, 3, &hint, &buflen, name, strlen(name) + 1) < 0)
+ 		goto error;
+	return (hint);
+error:
+	hint = GetDefaultEncoding();
+	return (hint);
+}
+
